/**
 * @file boost/mirror/meta_container.hpp
 * @brief Implementation of container registering and reflection
 *
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_MIRROR_META_CONTAINER_1104181551_HPP
#define BOOST_MIRROR_META_CONTAINER_1104181551_HPP

#include <boost/mirror/mirror_fwd.hpp>
#include <boost/mirror/tag_dispatch.hpp>
#include <boost/mirror/cntnr_kind_tags.hpp>
#include <boost/mirror/cntnr_op_param_tags.hpp>
#include <boost/mirror/preprocessor.hpp>
#include <boost/mirror/auxiliary/scoped_named.hpp>
#include <boost/mirror/auxiliary/containers.hpp>
#include <string>
#include <type_traits>

BOOST_MIRROR_NAMESPACE_BEGIN

namespace _container {

template <typename Container>
struct _
{
        typedef std::integral_constant<int, 0> cntnr_count;
        typedef non_container_tag kind;
};

template <size_t Size>
struct _array_base
{
        static bool empty(void*)
        {
                return Size == 0;
        }

        static size_t size_hint(void*)
        {
                return Size;
        }

        typedef boost::mirror::mp::empty_range inserters;
        typedef boost::mirror::mp::empty_range erasers;
};

template <class ElementType, size_t Size>
struct _<ElementType[Size]>
{
        struct cntnr : _array_base<Size>
        {
                typedef ElementType elem_type;

                typedef boost::mirror::aux::meta_array_trav_fwd<
                        ElementType,
                        Size
                > default_traversal;

                typedef boost::mirror::mp::range<
                        default_traversal
                        // TODO: reverse traversal ?
                > traversals;

                typedef boost::mirror::mp::range<
                        // TODO: begin/end/at/... ?
                > locators;
        };
        static cntnr _cntnr(std::integral_constant<int, 0>);
        typedef std::integral_constant<int, 1> cntnr_count;
        typedef simple_container_tag kind;
};

} // namespace _container

template <class Container, int Index>
struct meta_container
{
private:
        typedef decltype(
                boost::mirror::_container::_<Container>::_cntnr(
                        std::integral_constant<int, Index>()
                )
        ) _base_cntnr;

        friend class default_traversal<meta_container>;
        friend class traversals<meta_container>;
        friend class locators<meta_container>;
        friend class inserters<meta_container>;
        friend class erasers<meta_container>;
public:
        typedef typename boost::mirror::_container::_<
                Container
        >::kind container_kind;

        typedef typename boost::mirror::reflected<
                typename _base_cntnr::elem_type
        >::type element_type;

        typedef typename boost::mirror::reflected<
                Container
        >::type scope;

        static bool empty(Container& container)
        {
                return _base_cntnr::empty(container);
        }

        static auto size_hint(Container& container) ->
        decltype(_base_cntnr::size_hint(container))
        {
                return _base_cntnr::size_hint(container);
        }

};

template <class Container>
struct container_kind<meta_type<Container> >
{
        typedef typename boost::mirror::_container::_<
                Container
        >::kind type;
};

template <class Container>
struct container_kind<meta_class<Container> >
 : public container_kind<meta_type<Container> >
{ };

template <class Container>
struct container_kind<meta_enum<Container> >
 : public container_kind<meta_type<Container> >
{ };

template <class Container>
struct containers<meta_type<Container> >
{
        typedef typename boost::mirror::mp::apply_on_seq_pack<
                boost::mirror::aux::cntnr_helper<Container>,
                typename boost::mirror::_container::_<Container>::cntnr_count
        >::type type;
};

template <class Container>
struct containers<meta_class<Container> >
 : public containers<meta_type<Container> >
{ };

template <class Container>
struct containers<meta_enum<Container> >
{
        typedef boost::mirror::mp::empty_range type;
};

template <class Container>
struct all_containers<meta_class<Container> >
{
        // TODO: inherited containers
        typedef typename boost::mirror::mp::apply_on_seq_pack<
                boost::mirror::aux::cntnr_helper<Container>,
                typename boost::mirror::_container::_<Container>::cntnr_count
        >::type type;
};

template <class Container, int Index>
struct default_traversal<meta_container<Container, Index> >
{
        typedef typename meta_container<
                Container,
                Index
        >::_base_cntnr::default_traversal type;
};

template <class Container, int Index>
struct traversals<meta_container<Container, Index> >
{
        typedef typename meta_container<
                Container,
                Index
        >::_base_cntnr::traversals type;
};

template <class Container, int Index>
struct locators<meta_container<Container, Index> >
{
        typedef typename meta_container<
                Container,
                Index
        >::_base_cntnr::locators type;
};

template <class Container, int Index>
struct inserters<meta_container<Container, Index> >
{
        typedef typename meta_container<
                Container,
                Index
        >::_base_cntnr::inserters type;
};

template <class Container, int Index>
struct erasers<meta_container<Container, Index> >
{
        typedef typename meta_container<
                Container,
                Index
        >::_base_cntnr::erasers type;
};

BOOST_MIRROR_NAMESPACE_END

#endif //include guard

